@@ -26,6 +26,7 @@ from google.cloud._helpers import _datetime_to_rfc3339
 from google.cloud._helpers import _NOW
 from google.cloud._helpers import _rfc3339_to_datetime
 from google.cloud.exceptions import NotFound
+from google.cloud.iam import Policy
 from google.cloud.iterator import HTTPIterator
 from google.cloud.storage._helpers import _PropertyMixin
 from google.cloud.storage._helpers import _scalar_property
@@ -33,6 +34,7 @@ from google.cloud.storage._helpers import _validate_name
 from google.cloud.storage.acl import BucketACL
 from google.cloud.storage.acl import DefaultObjectACL
 from google.cloud.storage.blob import Blob
+from google.cloud.storage.blob import _get_encryption_headers
 
 
 def _blobs_page_start(iterator, page, response):
@@ -102,7 +104,7 @@ class Bucket(_PropertyMixin):
     )
     """Allowed values for :attr:`storage_class`.
 
-    See:
+    See
     https://cloud.google.com/storage/docs/json_api/v1/buckets#storageClass
     https://cloud.google.com/storage/docs/storage-classes
     """
@@ -115,7 +117,7 @@ class Bucket(_PropertyMixin):
         self._default_object_acl = DefaultObjectACL(self)
 
     def __repr__(self):
-        return '<Bucket: %s>' % self.name
+        return '<Bucket: %s>' % (self.name,)
 
     @property
     def client(self):
@@ -227,12 +229,12 @@ class Bucket(_PropertyMixin):
 
         return self.path_helper(self.name)
 
-    def get_blob(self, blob_name, client=None):
+    def get_blob(self, blob_name, client=None, encryption_key=None, **kwargs):
         """Get a blob object by name.
 
         This will return None if the blob doesn't exist:
 
-        .. literalinclude:: storage_snippets.py
+        .. literalinclude:: snippets.py
           :start-after: [START get_blob]
           :end-before: [END get_blob]
 
@@ -244,14 +246,27 @@ class Bucket(_PropertyMixin):
         :param client: Optional. The client to use.  If not passed, falls back
                        to the ``client`` stored on the current bucket.
 
+        :type encryption_key: bytes
+        :param encryption_key:
+            Optional 32 byte encryption key for customer-supplied encryption.
+            See
+            https://cloud.google.com/storage/docs/encryption#customer-supplied.
+
+        :type kwargs: dict
+        :param kwargs: Keyword arguments to pass to the
+                       :class:`~google.cloud.storage.blob.Blob` constructor.
+
         :rtype: :class:`google.cloud.storage.blob.Blob` or None
         :returns: The blob object if it exists, otherwise None.
         """
         client = self._require_client(client)
-        blob = Blob(bucket=self, name=blob_name)
+        blob = Blob(bucket=self, name=blob_name, encryption_key=encryption_key,
+                    **kwargs)
         try:
+            headers = _get_encryption_headers(encryption_key)
             response = client._connection.api_request(
-                method='GET', path=blob.path, _target_object=blob)
+                method='GET', path=blob.path, _target_object=blob,
+                headers=headers)
             # NOTE: We assume response.get('name') matches `blob_name`.
             blob._set_properties(response)
             # NOTE: This will not fail immediately in a batch. However, when
@@ -391,7 +406,7 @@ class Bucket(_PropertyMixin):
 
         For example:
 
-        .. literalinclude:: storage_snippets.py
+        .. literalinclude:: snippets.py
           :start-after: [START delete_blob]
           :end-before: [END delete_blob]
 
@@ -407,7 +422,7 @@ class Bucket(_PropertyMixin):
                  the exception, call ``delete_blobs``, passing a no-op
                  ``on_error`` callback, e.g.:
 
-        .. literalinclude:: storage_snippets.py
+        .. literalinclude:: snippets.py
             :start-after: [START delete_blobs]
             :end-before: [END delete_blobs]
 
@@ -526,9 +541,22 @@ class Bucket(_PropertyMixin):
     def cors(self):
         """Retrieve or set CORS policies configured for this bucket.
 
-        See: http://www.w3.org/TR/cors/ and
+        See http://www.w3.org/TR/cors/ and
              https://cloud.google.com/storage/docs/json_api/v1/buckets
 
+        .. note::
+
+           The getter for this property returns a list which contains
+           *copies* of the bucket's CORS policy mappings.  Mutating the list
+           or one of its dicts has no effect unless you then re-assign the
+           dict via the setter.  E.g.:
+
+           >>> policies = bucket.cors
+           >>> policies.append({'origin': '/foo', ...})
+           >>> policies[1]['maxAgeSeconds'] = 3600
+           >>> del policies[0]
+           >>> bucket.cors = policies
+
         :setter: Set CORS policies for this bucket.
         :getter: Gets the CORS policies for this bucket.
 
@@ -542,7 +570,7 @@ class Bucket(_PropertyMixin):
     def cors(self, entries):
         """Set CORS policies configured for this bucket.
 
-        See: http://www.w3.org/TR/cors/ and
+        See http://www.w3.org/TR/cors/ and
              https://cloud.google.com/storage/docs/json_api/v1/buckets
 
         :type entries: list of dictionaries
@@ -550,11 +578,52 @@ class Bucket(_PropertyMixin):
         """
         self._patch_property('cors', entries)
 
+    @property
+    def labels(self):
+        """Retrieve or set labels assigned to this bucket.
+
+        See
+        https://cloud.google.com/storage/docs/json_api/v1/buckets#labels
+
+        .. note::
+
+           The getter for this property returns a dict which is a *copy*
+           of the bucket's labels.  Mutating that dict has no effect unless
+           you then re-assign the dict via the setter.  E.g.:
+
+           >>> labels = bucket.labels
+           >>> labels['new_key'] = 'some-label'
+           >>> del labels['old_key']
+           >>> bucket.labels = labels
+
+        :setter: Set labels for this bucket.
+        :getter: Gets the labels for this bucket.
+
+        :rtype: :class:`dict`
+        :returns: Name-value pairs (string->string) labelling the bucket.
+        """
+        labels = self._properties.get('labels')
+        if labels is None:
+            return {}
+        return copy.deepcopy(labels)
+
+    @labels.setter
+    def labels(self, mapping):
+        """Set labels assigned to this bucket.
+
+        See
+        https://cloud.google.com/storage/docs/json_api/v1/buckets#labels
+
+        :type mapping: :class:`dict`
+        :param mapping: Name-value pairs (string->string) labelling the bucket.
+        """
+        self._patch_property('labels', copy.deepcopy(mapping))
+
     @property
     def etag(self):
         """Retrieve the ETag for the bucket.
 
-        See: https://tools.ietf.org/html/rfc2616#section-3.11 and
+        See https://tools.ietf.org/html/rfc2616#section-3.11 and
              https://cloud.google.com/storage/docs/json_api/v1/buckets
 
         :rtype: str or ``NoneType``
@@ -567,7 +636,7 @@ class Bucket(_PropertyMixin):
     def id(self):
         """Retrieve the ID for the bucket.
 
-        See: https://cloud.google.com/storage/docs/json_api/v1/buckets
+        See https://cloud.google.com/storage/docs/json_api/v1/buckets
 
         :rtype: str or ``NoneType``
         :returns: The ID of the bucket or ``None`` if the property is not
@@ -577,11 +646,27 @@ class Bucket(_PropertyMixin):
 
     @property
     def lifecycle_rules(self):
-        """Lifecycle rules configured for this bucket.
+        """Retrieve or set lifecycle rules configured for this bucket.
 
-        See: https://cloud.google.com/storage/docs/lifecycle and
+        See https://cloud.google.com/storage/docs/lifecycle and
              https://cloud.google.com/storage/docs/json_api/v1/buckets
 
+        .. note::
+
+           The getter for this property returns a list which contains
+           *copies* of the bucket's lifecycle rules mappings.  Mutating the
+           list or one of its dicts has no effect unless you then re-assign
+           the dict via the setter.  E.g.:
+
+           >>> rules = bucket.lifecycle_rules
+           >>> rules.append({'origin': '/foo', ...})
+           >>> rules[1]['rule']['action']['type'] = 'Delete'
+           >>> del rules[0]
+           >>> bucket.lifecycle_rules = rules
+
+        :setter: Set lifestyle rules for this bucket.
+        :getter: Gets the lifestyle rules for this bucket.
+
         :rtype: list(dict)
         :returns: A sequence of mappings describing each lifecycle rule.
         """
@@ -590,12 +675,20 @@ class Bucket(_PropertyMixin):
 
     @lifecycle_rules.setter
     def lifecycle_rules(self, rules):
+        """Set lifestyle rules configured for this bucket.
+
+        See https://cloud.google.com/storage/docs/lifecycle and
+             https://cloud.google.com/storage/docs/json_api/v1/buckets
+
+        :type entries: list of dictionaries
+        :param entries: A sequence of mappings describing each lifecycle rule.
+        """
         self._patch_property('lifecycle', {'rule': rules})
 
     location = _scalar_property('location')
     """Retrieve location configured for this bucket.
 
-    See: https://cloud.google.com/storage/docs/json_api/v1/buckets and
+    See https://cloud.google.com/storage/docs/json_api/v1/buckets and
     https://cloud.google.com/storage/docs/bucket-locations
 
     If the property is not set locally, returns ``None``.
@@ -606,7 +699,7 @@ class Bucket(_PropertyMixin):
     def get_logging(self):
         """Return info about access logging for this bucket.
 
-        See: https://cloud.google.com/storage/docs/access-logs#status
+        See https://cloud.google.com/storage/docs/access-logs#status
 
         :rtype: dict or None
         :returns: a dict w/ keys, ``logBucket`` and ``logObjectPrefix``
@@ -618,7 +711,7 @@ class Bucket(_PropertyMixin):
     def enable_logging(self, bucket_name, object_prefix=''):
         """Enable access logging for this bucket.
 
-        See: https://cloud.google.com/storage/docs/access-logs
+        See https://cloud.google.com/storage/docs/access-logs
 
         :type bucket_name: str
         :param bucket_name: name of bucket in which to store access logs
@@ -632,7 +725,7 @@ class Bucket(_PropertyMixin):
     def disable_logging(self):
         """Disable access logging for this bucket.
 
-        See: https://cloud.google.com/storage/docs/access-logs#disabling
+        See https://cloud.google.com/storage/docs/access-logs#disabling
         """
         self._patch_property('logging', None)
 
@@ -640,7 +733,7 @@ class Bucket(_PropertyMixin):
     def metageneration(self):
         """Retrieve the metageneration for the bucket.
 
-        See: https://cloud.google.com/storage/docs/json_api/v1/buckets
+        See https://cloud.google.com/storage/docs/json_api/v1/buckets
 
         :rtype: int or ``NoneType``
         :returns: The metageneration of the bucket or ``None`` if the property
@@ -654,7 +747,7 @@ class Bucket(_PropertyMixin):
     def owner(self):
         """Retrieve info about the owner of the bucket.
 
-        See: https://cloud.google.com/storage/docs/json_api/v1/buckets
+        See https://cloud.google.com/storage/docs/json_api/v1/buckets
 
         :rtype: dict or ``NoneType``
         :returns: Mapping of owner's role/ID. If the property is not set
@@ -666,7 +759,7 @@ class Bucket(_PropertyMixin):
     def project_number(self):
         """Retrieve the number of the project to which the bucket is assigned.
 
-        See: https://cloud.google.com/storage/docs/json_api/v1/buckets
+        See https://cloud.google.com/storage/docs/json_api/v1/buckets
 
         :rtype: int or ``NoneType``
         :returns: The project number that owns the bucket or ``None`` if the
@@ -680,7 +773,7 @@ class Bucket(_PropertyMixin):
     def self_link(self):
         """Retrieve the URI for the bucket.
 
-        See: https://cloud.google.com/storage/docs/json_api/v1/buckets
+        See https://cloud.google.com/storage/docs/json_api/v1/buckets
 
         :rtype: str or ``NoneType``
         :returns: The self link for the bucket or ``None`` if the property is
@@ -690,9 +783,12 @@ class Bucket(_PropertyMixin):
 
     @property
     def storage_class(self):
-        """Retrieve the storage class for the bucket.
+        """Retrieve or set the storage class for the bucket.
 
-        See: https://cloud.google.com/storage/docs/storage-classes
+        See https://cloud.google.com/storage/docs/storage-classes
+
+        :setter: Set the storage class for this bucket.
+        :getter: Gets the the storage class for this bucket.
 
         :rtype: str or ``NoneType``
         :returns: If set, one of "MULTI_REGIONAL", "REGIONAL",
@@ -705,7 +801,7 @@ class Bucket(_PropertyMixin):
     def storage_class(self, value):
         """Set the storage class for the bucket.
 
-        See: https://cloud.google.com/storage/docs/storage-classes
+        See https://cloud.google.com/storage/docs/storage-classes
 
         :type value: str
         :param value: one of "MULTI_REGIONAL", "REGIONAL", "NEARLINE",
@@ -719,7 +815,7 @@ class Bucket(_PropertyMixin):
     def time_created(self):
         """Retrieve the timestamp at which the bucket was created.
 
-        See: https://cloud.google.com/storage/docs/json_api/v1/buckets
+        See https://cloud.google.com/storage/docs/json_api/v1/buckets
 
         :rtype: :class:`datetime.datetime` or ``NoneType``
         :returns: Datetime object parsed from RFC3339 valid timestamp, or
@@ -733,9 +829,12 @@ class Bucket(_PropertyMixin):
     def versioning_enabled(self):
         """Is versioning enabled for this bucket?
 
-        See:  https://cloud.google.com/storage/docs/object-versioning for
+        See  https://cloud.google.com/storage/docs/object-versioning for
         details.
 
+        :setter: Update whether versioning is enabled for this bucket.
+        :getter: Query whether versioning is enabled for this bucket.
+
         :rtype: bool
         :returns: True if enabled, else False.
         """
@@ -746,7 +845,7 @@ class Bucket(_PropertyMixin):
     def versioning_enabled(self, value):
         """Enable versioning for this bucket.
 
-        See:  https://cloud.google.com/storage/docs/object-versioning for
+        See  https://cloud.google.com/storage/docs/object-versioning for
         details.
 
         :type value: convertible to boolean
@@ -757,7 +856,7 @@ class Bucket(_PropertyMixin):
     def configure_website(self, main_page_suffix=None, not_found_page=None):
         """Configure website-related properties.
 
-        See: https://cloud.google.com/storage/docs/hosting-static-website
+        See https://cloud.google.com/storage/docs/hosting-static-website
 
         .. note::
           This (apparently) only works
@@ -767,13 +866,13 @@ class Bucket(_PropertyMixin):
         If you want this bucket to host a website, just provide the name
         of an index page and a page to use when a blob isn't found:
 
-        .. literalinclude:: storage_snippets.py
+        .. literalinclude:: snippets.py
           :start-after: [START configure_website]
           :end-before: [END configure_website]
 
         You probably should also make the whole bucket public:
 
-        .. literalinclude:: storage_snippets.py
+        .. literalinclude:: snippets.py
             :start-after: [START make_public]
             :end-before: [END make_public]
 
@@ -803,6 +902,83 @@ class Bucket(_PropertyMixin):
         """
         return self.configure_website(None, None)
 
+    def get_iam_policy(self, client=None):
+        """Retrieve the IAM policy for the bucket.
+
+        See
+        https://cloud.google.com/storage/docs/json_api/v1/buckets/getIamPolicy
+
+        :type client: :class:`~google.cloud.storage.client.Client` or
+                      ``NoneType``
+        :param client: Optional. The client to use.  If not passed, falls back
+                       to the ``client`` stored on the current bucket.
+
+        :rtype: :class:`google.cloud.iam.Policy`
+        :returns: the policy instance, based on the resource returned from
+                  the ``getIamPolicy`` API request.
+        """
+        client = self._require_client(client)
+        info = client._connection.api_request(
+            method='GET',
+            path='%s/iam' % (self.path,),
+            _target_object=None)
+        return Policy.from_api_repr(info)
+
+    def set_iam_policy(self, policy, client=None):
+        """Update the IAM policy for the bucket.
+
+        See
+        https://cloud.google.com/storage/docs/json_api/v1/buckets/setIamPolicy
+
+        :type policy: :class:`google.cloud.iam.Policy`
+        :param policy: policy instance used to update bucket's IAM policy.
+
+        :type client: :class:`~google.cloud.storage.client.Client` or
+                      ``NoneType``
+        :param client: Optional. The client to use.  If not passed, falls back
+                       to the ``client`` stored on the current bucket.
+
+        :rtype: :class:`google.cloud.iam.Policy`
+        :returns: the policy instance, based on the resource returned from
+                  the ``setIamPolicy`` API request.
+        """
+        client = self._require_client(client)
+        resource = policy.to_api_repr()
+        resource['resourceId'] = self.path
+        info = client._connection.api_request(
+            method='PUT',
+            path='%s/iam' % (self.path,),
+            data=resource,
+            _target_object=None)
+        return Policy.from_api_repr(info)
+
+    def test_iam_permissions(self, permissions, client=None):
+        """API call:  test permissions
+
+        See
+        https://cloud.google.com/storage/docs/json_api/v1/buckets/testIamPermissions
+
+        :type permissions: list of string
+        :param permissions: the permissions to check
+
+        :type client: :class:`~google.cloud.storage.client.Client` or
+                      ``NoneType``
+        :param client: Optional. The client to use.  If not passed, falls back
+                       to the ``client`` stored on the current bucket.
+
+        :rtype: list of string
+        :returns: the permissions returned by the ``testIamPermissions`` API
+                  request.
+        """
+        client = self._require_client(client)
+        query = {'permissions': permissions}
+        path = '%s/iam/testPermissions' % (self.path,)
+        resp = client._connection.api_request(
+            method='GET',
+            path=path,
+            query_params=query)
+        return resp.get('permissions', [])
+
     def make_public(self, recursive=False, future=False, client=None):
         """Make a bucket public.
 
@@ -861,7 +1037,7 @@ class Bucket(_PropertyMixin):
 
         For example:
 
-        .. literalinclude:: storage_snippets.py
+        .. literalinclude:: snippets.py
             :start-after: [START policy_document]
             :end-before: [END policy_document]
 
@@ -890,8 +1066,9 @@ class Bucket(_PropertyMixin):
         credentials = client._base_connection.credentials
 
         if not isinstance(credentials, google.auth.credentials.Signing):
-            auth_uri = ('http://google-cloud-python.readthedocs.io/en/latest/'
-                        'google-cloud-auth.html#setting-up-a-service-account')
+            auth_uri = ('https://google-cloud-python.readthedocs.io/en/latest/'
+                        'core/auth.html?highlight=authentication#setting-up-'
+                        'a-service-account')
             raise AttributeError(
                 'you need a private key to sign credentials.'
                 'the credentials you are currently using %s '
