@@ -17,7 +17,7 @@ import tempfile
 import time
 import unittest
 
-import httplib2
+import requests
 import six
 
 from google.cloud import exceptions
@@ -28,9 +28,6 @@ from test_utils.retry import RetryErrors
 from test_utils.system import unique_resource_id
 
 
-HTTP = httplib2.Http()
-
-
 def _bad_copy(bad_request):
     """Predicate: pass only exceptions for a failed copyTo."""
     err_msg = bad_request.message
@@ -246,6 +243,7 @@ class TestStorageWriteFiles(TestStorageFiles):
 
 
 class TestUnicode(unittest.TestCase):
+
     def test_fetch_object_and_check_content(self):
         client = storage.Client()
         bucket = client.bucket('storage-library-test-bucket')
@@ -256,8 +254,8 @@ class TestUnicode(unittest.TestCase):
         # Normalization Form D: an ASCII e followed by U+0301 combining
         # character; URL should end with Caf%C3%A9
         test_data = {
-            u'Caf\u00e9'.encode('utf-8'): b'Normalization Form C',
-            u'Cafe\u0301'.encode('utf-8'): b'Normalization Form D',
+            u'Caf\u00e9': b'Normalization Form C',
+            u'Cafe\u0301': b'Normalization Form D',
         }
         for blob_name, file_contents in test_data.items():
             blob = bucket.blob(blob_name)
@@ -425,9 +423,9 @@ class TestStorageSignURLs(TestStorageFiles):
         signed_url = blob.generate_signed_url(expiration, method='GET',
                                               client=Config.CLIENT)
 
-        response, content = HTTP.request(signed_url, method='GET')
-        self.assertEqual(response.status, 200)
-        self.assertEqual(content, self.LOCAL_FILE)
+        response = requests.get(signed_url)
+        self.assertEqual(response.status_code, 200)
+        self.assertEqual(response.content, self.LOCAL_FILE)
 
     def test_create_signed_delete_url(self):
         blob = self.bucket.blob('LogoToSign.jpg')
@@ -436,9 +434,9 @@ class TestStorageSignURLs(TestStorageFiles):
                                                      method='DELETE',
                                                      client=Config.CLIENT)
 
-        response, content = HTTP.request(signed_delete_url, method='DELETE')
-        self.assertEqual(response.status, 204)
-        self.assertEqual(content, b'')
+        response = requests.request('DELETE', signed_delete_url)
+        self.assertEqual(response.status_code, 204)
+        self.assertEqual(response.content, b'')
 
         # Check that the blob has actually been deleted.
         self.assertFalse(blob.exists())
