@@ -167,6 +167,7 @@ class Test_Bucket(unittest.TestCase):
             "condition": {"age": 365}
         }]
         LOCATION = 'eu'
+        LABELS = {'color': 'red', 'flavor': 'cherry'}
         STORAGE_CLASS = 'NEARLINE'
         DATA = {
             'name': BUCKET_NAME,
@@ -175,6 +176,7 @@ class Test_Bucket(unittest.TestCase):
             'location': LOCATION,
             'storageClass': STORAGE_CLASS,
             'versioning': {'enabled': True},
+            'labels': LABELS,
         }
         connection = _Connection(DATA)
         client = _Client(connection, project=PROJECT)
@@ -184,6 +186,7 @@ class Test_Bucket(unittest.TestCase):
         bucket.location = LOCATION
         bucket.storage_class = STORAGE_CLASS
         bucket.versioning_enabled = True
+        bucket.labels = LABELS
         bucket.create()
 
         kw, = connection._requested
@@ -242,6 +245,29 @@ class Test_Bucket(unittest.TestCase):
         self.assertEqual(kw['method'], 'GET')
         self.assertEqual(kw['path'], '/b/%s/o/%s' % (NAME, BLOB_NAME))
 
+    def test_get_blob_hit_with_kwargs(self):
+        from google.cloud.storage.blob import _get_encryption_headers
+
+        NAME = 'name'
+        BLOB_NAME = 'blob-name'
+        CHUNK_SIZE = 1024 * 1024
+        KEY = b'01234567890123456789012345678901'  # 32 bytes
+
+        connection = _Connection({'name': BLOB_NAME})
+        client = _Client(connection)
+        bucket = self._make_one(name=NAME)
+        blob = bucket.get_blob(
+            BLOB_NAME, client=client, encryption_key=KEY, chunk_size=CHUNK_SIZE
+        )
+        self.assertIs(blob.bucket, bucket)
+        self.assertEqual(blob.name, BLOB_NAME)
+        kw, = connection._requested
+        self.assertEqual(kw['method'], 'GET')
+        self.assertEqual(kw['path'], '/b/%s/o/%s' % (NAME, BLOB_NAME))
+        self.assertEqual(kw['headers'], _get_encryption_headers(KEY))
+        self.assertEqual(blob.chunk_size, CHUNK_SIZE)
+        self.assertEqual(blob._encryption_key, KEY)
+
     def test_list_blobs_defaults(self):
         NAME = 'name'
         connection = _Connection({'items': []})
@@ -663,6 +689,27 @@ class Test_Bucket(unittest.TestCase):
         self.assertEqual(bucket.cors, [CORS_ENTRY])
         self.assertTrue('cors' in bucket._changes)
 
+    def test_labels_getter(self):
+        NAME = 'name'
+        LABELS = {'color': 'red', 'flavor': 'cherry'}
+        properties = {'labels': LABELS}
+        bucket = self._make_one(name=NAME, properties=properties)
+        labels = bucket.labels
+        self.assertEqual(labels, LABELS)
+        # Make sure it was a copy, not the same object.
+        self.assertIsNot(labels, LABELS)
+
+    def test_labels_setter(self):
+        NAME = 'name'
+        LABELS = {'color': 'red', 'flavor': 'cherry'}
+        bucket = self._make_one(name=NAME)
+
+        self.assertEqual(bucket.labels, {})
+        bucket.labels = LABELS
+        self.assertEqual(bucket.labels, LABELS)
+        self.assertIsNot(bucket._properties['labels'], LABELS)
+        self.assertIn('labels', bucket._changes)
+
     def test_get_logging_w_prefix(self):
         NAME = 'name'
         LOG_BUCKET = 'logs'
@@ -866,6 +913,135 @@ class Test_Bucket(unittest.TestCase):
         bucket.disable_website()
         self.assertEqual(bucket._properties, UNSET)
 
+    def test_get_iam_policy(self):
+        from google.cloud.storage.iam import STORAGE_OWNER_ROLE
+        from google.cloud.storage.iam import STORAGE_EDITOR_ROLE
+        from google.cloud.storage.iam import STORAGE_VIEWER_ROLE
+        from google.cloud.iam import Policy
+
+        NAME = 'name'
+        PATH = '/b/%s' % (NAME,)
+        ETAG = 'DEADBEEF'
+        VERSION = 17
+        OWNER1 = 'user:phred@example.com'
+        OWNER2 = 'group:cloud-logs@google.com'
+        EDITOR1 = 'domain:google.com'
+        EDITOR2 = 'user:phred@example.com'
+        VIEWER1 = 'serviceAccount:1234-abcdef@service.example.com'
+        VIEWER2 = 'user:phred@example.com'
+        RETURNED = {
+            'resourceId': PATH,
+            'etag': ETAG,
+            'version': VERSION,
+            'bindings': [
+                {'role': STORAGE_OWNER_ROLE, 'members': [OWNER1, OWNER2]},
+                {'role': STORAGE_EDITOR_ROLE, 'members': [EDITOR1, EDITOR2]},
+                {'role': STORAGE_VIEWER_ROLE, 'members': [VIEWER1, VIEWER2]},
+            ],
+        }
+        EXPECTED = {
+            binding['role']: set(binding['members'])
+            for binding in RETURNED['bindings']}
+        connection = _Connection(RETURNED)
+        client = _Client(connection, None)
+        bucket = self._make_one(client=client, name=NAME)
+
+        policy = bucket.get_iam_policy()
+
+        self.assertIsInstance(policy, Policy)
+        self.assertEqual(policy.etag, RETURNED['etag'])
+        self.assertEqual(policy.version, RETURNED['version'])
+        self.assertEqual(dict(policy), EXPECTED)
+
+        kw = connection._requested
+        self.assertEqual(len(kw), 1)
+        self.assertEqual(kw[0]['method'], 'GET')
+        self.assertEqual(kw[0]['path'], '%s/iam' % (PATH,))
+
+    def test_set_iam_policy(self):
+        import operator
+        from google.cloud.storage.iam import STORAGE_OWNER_ROLE
+        from google.cloud.storage.iam import STORAGE_EDITOR_ROLE
+        from google.cloud.storage.iam import STORAGE_VIEWER_ROLE
+        from google.cloud.iam import Policy
+
+        NAME = 'name'
+        PATH = '/b/%s' % (NAME,)
+        ETAG = 'DEADBEEF'
+        VERSION = 17
+        OWNER1 = 'user:phred@example.com'
+        OWNER2 = 'group:cloud-logs@google.com'
+        EDITOR1 = 'domain:google.com'
+        EDITOR2 = 'user:phred@example.com'
+        VIEWER1 = 'serviceAccount:1234-abcdef@service.example.com'
+        VIEWER2 = 'user:phred@example.com'
+        BINDINGS = [
+            {'role': STORAGE_OWNER_ROLE, 'members': [OWNER1, OWNER2]},
+            {'role': STORAGE_EDITOR_ROLE, 'members': [EDITOR1, EDITOR2]},
+            {'role': STORAGE_VIEWER_ROLE, 'members': [VIEWER1, VIEWER2]},
+        ]
+        RETURNED = {
+            'etag': ETAG,
+            'version': VERSION,
+            'bindings': BINDINGS,
+        }
+        policy = Policy()
+        for binding in BINDINGS:
+            policy[binding['role']] = binding['members']
+
+        connection = _Connection(RETURNED)
+        client = _Client(connection, None)
+        bucket = self._make_one(client=client, name=NAME)
+
+        returned = bucket.set_iam_policy(policy)
+
+        self.assertEqual(returned.etag, ETAG)
+        self.assertEqual(returned.version, VERSION)
+        self.assertEqual(dict(returned), dict(policy))
+
+        kw = connection._requested
+        self.assertEqual(len(kw), 1)
+        self.assertEqual(kw[0]['method'], 'PUT')
+        self.assertEqual(kw[0]['path'], '%s/iam' % (PATH,))
+        sent = kw[0]['data']
+        self.assertEqual(sent['resourceId'], PATH)
+        self.assertEqual(len(sent['bindings']), len(BINDINGS))
+        key = operator.itemgetter('role')
+        for found, expected in zip(
+            sorted(sent['bindings'], key=key),
+            sorted(BINDINGS, key=key)):
+            self.assertEqual(found['role'], expected['role'])
+            self.assertEqual(
+                sorted(found['members']), sorted(expected['members']))
+
+    def test_test_iam_permissions(self):
+        from google.cloud.storage.iam import STORAGE_OBJECTS_LIST
+        from google.cloud.storage.iam import STORAGE_BUCKETS_GET
+        from google.cloud.storage.iam import STORAGE_BUCKETS_UPDATE
+
+        NAME = 'name'
+        PATH = '/b/%s' % (NAME,)
+        PERMISSIONS = [
+            STORAGE_OBJECTS_LIST,
+            STORAGE_BUCKETS_GET,
+            STORAGE_BUCKETS_UPDATE,
+        ]
+        ALLOWED = PERMISSIONS[1:]
+        RETURNED = {'permissions': ALLOWED}
+        connection = _Connection(RETURNED)
+        client = _Client(connection, None)
+        bucket = self._make_one(client=client, name=NAME)
+
+        allowed = bucket.test_iam_permissions(PERMISSIONS)
+
+        self.assertEqual(allowed, ALLOWED)
+
+        kw = connection._requested
+        self.assertEqual(len(kw), 1)
+        self.assertEqual(kw[0]['method'], 'GET')
+        self.assertEqual(kw[0]['path'], '%s/iam/testPermissions' % (PATH,))
+        self.assertEqual(kw[0]['query_params'], {'permissions': PERMISSIONS})
+
     def test_make_public_defaults(self):
         from google.cloud.storage.acl import _ACLEntity
 
